Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

entity Bus_Management is 
	port( 	
			clk					: in std_logic;
			nreset				: in std_logic;
			Bus_Req_Proc_1		: in std_logic;
			Data_Proc_1			: in std_logic_vector(15 downto 0);
			Addr_Proc_1			: in std_logic_vector(15 downto 0);
			WR_Proc_1			: in std_logic;
			Bus_Req_Proc_2		: in std_logic;
			Data_Proc_2			: in std_logic_vector(15 downto 0);
			Addr_Proc_2			: in std_logic_vector(15 downto 0);
			WR_Proc_2			: in std_logic;
			
			Bus_Permit_Proc_1	: out std_logic;
			Bus_Permit_Proc_2	: out std_logic;
			Data_to_Mem			: out std_logic_vector(15 downto 0);
			Addr_to_Mem			: out std_logic_vector(15 downto 0);
			WR_to_Mem			: out std_logic		
		);
	end Bus_Management;
	
architecture main of Bus_Management is	


	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, permit1, permit2);
	signal CS, NS  : STATE_TYPE;

--======================================================================
--               B  E  G  I  N  
------------------------------------------------------------------------       
begin


-- State Machine define initialization ---------------------------
	sm_reg: process (clk, nreset)	
	begin
		if(nreset = '0')            then CS <= idle;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: 
	process 
	(CS, Bus_Req_Proc_1, Bus_Req_Proc_2, 
	 Data_Proc_1, Addr_Proc_1, WR_Proc_1,
	 Data_Proc_2, Addr_Proc_2, WR_Proc_2 )
	begin
		-- default signal values
		Bus_Permit_Proc_1	<= '0';
		Bus_Permit_Proc_2	<= '0';
		Data_to_Mem			<= Data_Proc_1;
		Addr_to_Mem			<= Addr_Proc_1;
		WR_to_Mem			<= WR_Proc_1;	
                            
		-------------------------
		case CS is
			when idle   =>
				if   (Bus_Req_Proc_1 = '1') then 	NS <= permit1;
				elsif(Bus_Req_Proc_2 = '1') then 	NS <= permit2;
				else								NS <= idle;
				end if;
			    
			when permit1  =>						
				Bus_Permit_Proc_1	<= '1';
				Data_to_Mem			<= Data_Proc_1;
				Addr_to_Mem			<= Addr_Proc_1;
				WR_to_Mem			<= WR_Proc_1;	
				
				if  (Bus_Req_Proc_1 = '1') then 	NS <= permit1;
				else								NS <= idle;
				end if;
			    
			when permit2  =>						
				Bus_Permit_Proc_2	<= '1';
				Data_to_Mem			<= Data_Proc_2;
				Addr_to_Mem			<= Addr_Proc_2;
				WR_to_Mem			<= WR_Proc_2;	
				
				if  (Bus_Req_Proc_2 = '1') then 	NS <= permit2;
				else								NS <= idle;
				end if;
			    
			when others  =>						NS <= idle;
		end case; 
				
	end process;

end;