Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity Capture_ZC_Time is
	port(clk        		: in std_logic;
		 nreset     		: in std_logic;
		 Initialize		 	: in std_logic;
		 Trigger_A			: in std_logic;
		 Trigger_B			: in std_logic;
		 ZC_Time_A  		: in std_logic_vector(15 downto 0);
		 ZC_Time_B  		: in std_logic_vector(15 downto 0);
		 Delta_Time_A  		: in std_logic_vector(15 downto 0);
		 Delta_Time_B  		: in std_logic_vector(15 downto 0);
		 
		 Freq_Trigger		: out std_logic;
		 ZC_Time_out		: out std_logic_vector(15 downto 0);
		 Delta_Time_out		: out std_logic_vector(15 downto 0);
		 Delta_Time_mon_OB	: out std_logic_vector(13 downto 0)
		);
end Capture_ZC_Time;
	
architecture main of Capture_ZC_Time is

signal init			: std_logic;
signal trigA		: std_logic;
signal trigB		: std_logic;
signal ltchA		: std_logic;
signal ltchB		: std_logic;

signal ltchTrig		: std_logic; 
signal clrTrig 		: std_logic; 

signal delta_2s		: std_logic_vector(15 downto 0);
               
	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, watch, latch_A, latch_B, done);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 
sync_INIT: led
port map(sig_in => Initialize, clk => clk, clrn => nreset, pls_out => init);
	
sync_trigA: led
port map(sig_in => Trigger_A, clk => clk, clrn => nreset, pls_out => trigA);

sync_trigB: led
port map(sig_in => Trigger_B, clk => clk, clrn => nreset, pls_out => trigB);
	
---------------------------------------------------------------------
LATCH_TRIG: process(clk, nreset, ltchTrig, clrTrig)
begin
	if  (nreset = '0')        then Freq_Trigger <= '0';
	elsif rising_edge(clk)    then
		if   (ltchTrig = '1') then Freq_Trigger <= '1';
		elsif(clrTrig  = '1') then Freq_Trigger <= '0';
		end if;
	end if;
end process;
  
LATCH_ZC_TIME: process(clk, nreset, ltchA, ltchB, ZC_Time_A, ZC_Time_B)
begin
	if  (nreset = '0')     then ZC_Time_out <= (others => '0');
	elsif rising_edge(clk) then
		if   (ltchA = '1') then ZC_Time_out <= ZC_Time_A;
		elsif(ltchB = '1') then ZC_Time_out <= ZC_Time_B;
		end if;
	end if;
end process;
 
LATCH_DELTA_TIME: process(clk, nreset, ltchA, ltchB, Delta_Time_A, Delta_Time_B)
begin
	if  (nreset = '0')     then delta_2s <= (others => '0');
	elsif rising_edge(clk) then
		if   (ltchA = '1') then delta_2s <= Delta_Time_A;
		elsif(ltchB = '1') then delta_2s <= Delta_Time_B;
		end if;
	end if;
end process;

----------------------------------------------------------
 
Delta_Time_out 		<= delta_2s; 
Delta_Time_mon_OB	<= NOT delta_2s(15) & delta_2s(12 downto 0);

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if (nreset='0')             then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, init, trigA, trigB)
begin
	ltchA		<= '0';
	ltchB		<= '0';	
	
	ltchTrig 	<= '0';
	clrTrig  	<= '0';
	
	case CS is
	when idle =>
		clrTrig  <= '1';
		if(init = '1') then			NS <= watch;
		else						NS <= idle;
		end if;
		
	when watch =>
		if   (trigA = '1') then		NS <= latch_A;
		elsif(trigB = '1') then		NS <= latch_B;
		else						NS <= watch;
		end if;
			
	when latch_A =>					NS <= done;
		ltchA		<= '1';
		ltchTrig 	<= '1';

	when latch_B =>					NS <= done;
		ltchB		<= '1';
		ltchTrig	<= '1';

	when done	=>					NS <= idle;
	
	when others =>					NS <= idle;
	end case;			
end process;

end main;