Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities


--Version 4 breaks the EXTERNAL / MADC1 Switch logic out from the 
-- Accel_PL_Ena and Start / Stop logic.

entity Freq_Trigger_Gate is
	port(clk        		: in std_logic;
		 nreset     		: in std_logic;
		 Beam_Cyc_Start		: in std_logic;
		 MIPL_PD_Data_OB	: in std_logic_vector(15 downto 0);
		 Data_Strobe		: in std_logic;
		 
 		 Gate_Freq_Trig		: out std_logic

		);
end Freq_Trigger_Gate;
	
architecture main of Freq_Trigger_Gate is

signal start			: std_logic;
signal strobe			: std_logic;

signal timer			: std_logic_vector(23 downto 0);
signal timer_ena		: std_logic;

signal data_OB			: std_logic_vector(15 downto 0);

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, test_for_gate, gate_trigger, end_cycle);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 
RISING_EDGE_START: led
port map(sig_in => Beam_Cyc_Start, clk => clk, clrn => nreset, pls_out => start);

RISING_EDGE_STROBE: led
port map(sig_in => Data_Strobe, clk => clk, clrn => nreset, pls_out => strobe);


PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '0') then 	timer <= (others => '0');
		else						timer <= timer + 1;
		end if;
	end if;
end process;

LATCH_DATA_ON_STROBE: process(clk, nreset, strobe, MIPL_PD_Data_OB)
begin
	if  (nreset = '0') then 	data_OB <= (others => '0');
	elsif rising_edge(clk) then
		if  (strobe = '1') then
								data_OB <= MIPL_PD_Data_OB;
		end if;
	end if;
end process;


----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, timer, start, data_OB)
begin
	-- default outputs --
	timer_ena 		<= '1'; -- not enabled only when in idle state
	Gate_Freq_Trig	<= '0';
	
	case CS is
		when idle   =>
			timer_ena		<= '0';  
			if (start = '1') then 				NS <= test_for_gate;	
			else								NS <= idle;
			end if;

		when test_for_gate =>
			if   (data_OB > X"C000") 	then	NS <= gate_trigger; 
			elsif(timer = X"30D400")    then	NS <= end_cycle; -- time out at 40 ms
			else								NS <= test_for_gate;
			end if;
		
		when gate_trigger  =>
			Gate_Freq_Trig	<= '1';

			if   (timer = X"30D400")     then	NS <= end_cycle; -- time out at 40 ms
			else								NS <= gate_trigger;
			end if;

		when end_cycle  =>						NS <= idle;
			
		when others =>							NS <= idle;
	end case; 
			
end process;

end main;