
Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities



entity Integral_Feedback is
	port(clk        		: in std_logic;
		 nreset     		: in std_logic;
		 Ref_minus_Pherr	: in std_logic_vector(31 downto 0); 
		 Update_Strobe		: in std_logic;
		 Run_MIPL 			: in std_logic;
		 Integ_Gain_OB  	: in std_logic_vector(13 downto 0);
		 Intg_Gain_Ramp		: in std_logic_vector(13 downto 0);
		 
		 Intg_FeedBack_Out_2s  : out std_logic_vector(31 downto 0);
		 Intg_Output_Monitor   : out std_logic_vector(13 downto 0)
		 
		);
end Integral_Feedback;
	
architecture main of Integral_Feedback is

-- Signed Value of inputs ----------------------------------
	signal ref_minus_pherr_2s	: std_logic_vector(15 downto 0);
	signal Integ_Gain_2s		: std_logic_vector(15 downto 0);
	signal intg_gain_ramp_2s	: std_logic_vector(15 downto 0);
	signal latch_ig_ramp_2s		: std_logic_vector(15 downto 0);
	signal scaled_Intg_Gain_2s	: std_logic_vector(31 downto 0);
	
	signal integ_acc_cnt		: std_logic_vector(7 downto 0);
	signal inc_acc_cnt			: std_logic;
	signal clr_acc_cnt			: std_logic;
	
	
	signal product_2s	: std_logic_vector(31 downto 0);
	signal sum1_2s		: std_logic_vector(31 downto 0);
	signal acc_reg		: std_logic_vector(31 downto 0);

-------------------------------------------------------------
	signal  run			: std_logic;
	signal  upd			: std_logic;
	
	signal	get_fb		: std_logic;	
	signal  clr_acc		: std_logic;

----------------------------------------------------------------
	component mult16signed
	PORT
	(
		dataa		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0)
	);
	end component;
	
	component addsub16signed
	PORT
	(
		add_sub		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (15 DOWNTO 0)
	);
	end component;
	
	component addsub32signed
	PORT
	(
		add_sub		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0)
	);
	end component;

----------------------------------------------------------------

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, wait_1, get_feedback);
	signal CS, NS  : STATE_TYPE;
	

-- ## MAIN #########################################################
begin 

RESYNC_RUN: process(clk, nreset, Run_MIPL)
begin
	if   (nreset = '0')    then run <= '0';
	elsif rising_edge(clk) then run <= Run_MIPL;
	end if;
end process;

sync_UPDATE: led
port map(sig_in => Update_Strobe, clk => clk, clrn => nreset, pls_out => upd);

-- Convert Offset Binary Values to 2's Complement ------------------
Integ_Gain_2s 		<= (15 downto 13 => not Integ_Gain_OB(13)) & Integ_Gain_OB(12 downto 0);
intg_gain_ramp_2s	<= (15 downto 13 => not Intg_Gain_Ramp(13)) & Intg_Gain_Ramp(12 downto 0);

-- Gate off Phase to Reference Error Until MIPL --------------------
ENABLE_MIPL_FEEDBACK:
	process (clk, nreset, run, Ref_minus_Pherr)
	begin
		if   (nreset = '0') then 
			ref_minus_pherr_2s <= (others => '0');
		elsif rising_edge(clk) then
			if   (run = '0') then 	
				ref_minus_pherr_2s <= (others => '0');
			else 
				ref_minus_pherr_2s <= (not Ref_minus_Pherr(31)) & Ref_minus_Pherr(29 downto 15);
			end if;
		end if;
	end process;
		
		   
	LATCH_RAMP_GAINS:
	process(clk, nreset, get_fb, intg_gain_ramp_2s)
	begin
		if  (nreset = '0')     	then latch_ig_ramp_2s <= (others => '0');
		elsif rising_edge(clk) 	then
			if   (get_fb = '1')  	then latch_ig_ramp_2s <= intg_gain_ramp_2s;
			end if;
		end if;
	end process;

	INTGxINTG_RAMP : mult16signed -- this is the ramped integral gain
	port map (
				dataa	 => latch_ig_ramp_2s,
				datab	 => Integ_Gain_2s,
				result	 => scaled_Intg_Gain_2s(31 downto 0)
			 );
			   
	G2xFEEDBACK : mult16signed 
	port map (
				dataa	 => ref_minus_pherr_2s, 
				datab	 => scaled_Intg_Gain_2s(31) & scaled_Intg_Gain_2s(29 downto 15),
				result	 => product_2s(31 downto 0)
			 );
			 
	ACCUMULATOR_INPUT : addsub32signed 
	port map (
				add_sub	 => '1',
				dataa	 => product_2s(31 downto 0),
				datab	 => acc_reg(31 downto 0),
				result	 => sum1_2s(31 downto 0)
			 );
			
	ACCUMULATOR_REGISTER:
	process(clk, nreset, get_fb, clr_acc, sum1_2s)
	begin
		if  (nreset = '0')       then acc_reg <= (others => '0');
		elsif rising_edge(clk)   then
			if   (get_fb = '1')  then acc_reg <= sum1_2s;
			elsif(clr_acc = '1') then acc_reg <= (others => '0');
			end if;
		end if;
	end process;
	
-- State Machine Process definition ---------------------------
	sm_reg: process (clk, nreset)	
	begin
		if (nreset='0')			then CS <= idle;
		elsif rising_edge(clk)	then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: 
	process (CS, run, upd)
	begin
		-- defaults -------
		get_fb 		<= '0'; 
		clr_acc		<= '0';
		
		case CS is
			when idle =>				NS <= wait_1;
				clr_acc		<= '1';
				if (run = '1') then		NS <= wait_1;
				else					NS <= idle;
				end if;

			when wait_1 => 
				if(run = '0') then		NS <= idle;
				elsif(upd = '1') then	NS <= get_feedback;
				else					NS <= wait_1;
				end if;
			
			when get_feedback => 		NS <= wait_1;
				get_fb 		<= '1';	-- latch sum1_2s into the accumulator
			when others => 				NS <= idle;
		end case; 
				
	end process; -- sm_combinatorial

	
-- Syncronize Output ----------------------------

	process(clk, nreset, upd, acc_reg )
	begin
		if   (nreset = '0') then 
				Intg_FeedBack_Out_2s <= (others => '0');
				Intg_Output_Monitor  <= (others => '0');
		elsif rising_edge(clk) then
			if   (upd = '1') then 
				Intg_FeedBack_Out_2s <= acc_reg;
				Intg_Output_Monitor <= not acc_reg(31) & acc_reg(29 downto 17);
			end if;
		end if;
	end process;

	
end main;

