
Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

-- Version 3 gates of updates to the FPGA DAC except during the MIPL
-- intreval.  The DAC is set to zero at End_MIPL.

-- Version 5 changed to 32 bit and higher math with a 32 bit 
--  Reg_minus_Pherr in.  Also removed the Integral gain interval adjust.

entity Ramped_Gain is
	port(clk        		: in std_logic;
		 nreset     		: in std_logic;
		 Ref_minus_Pherr	: in std_logic_vector(31 downto 0); 
		 Run_MIPL 			: in std_logic;
		 MIPL_Gain			: in std_logic_vector(13 downto 0);
		 Gain_Ramp			: in std_logic_vector(13 downto 0);
		 
		 Output_2s			: out std_logic_vector(31 downto 0);
		 Monitor_Out		: out std_logic_vector(13 downto 0)
		 
		);
end Ramped_Gain;
	
architecture main of Ramped_Gain is

-- Signed Value of inputs ----------------------------------
	signal ref_minus_pherr_2s	: std_logic_vector(31 downto 0);
	signal gain_ramp_2s			: std_logic_vector(13 downto 0);
	signal MIPL_Gain_2s			: std_logic_vector(13 downto 0);
		
	signal ramp_2s				: std_logic_vector(45 downto 0);
	signal product0_2s	: std_logic_vector(45 downto 0);

-------------------------------------------------------------
	signal  run			: std_logic;
	
----------------------------------------------------------------
	component mult_32x14
	PORT
	(
		dataa		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (45 DOWNTO 0)
	);
	end component;


-- ## MAIN #########################################################
begin 

RESYNC_RUN: process(clk, nreset, Run_MIPL)
begin
	if   (nreset = '0')    then run <= '0';
	elsif rising_edge(clk) then run <= Run_MIPL;
	end if;
end process;

-- Convert Offset Binary Values to 2's Complement ------------------
MIPL_Gain_2s 	<= not MIPL_Gain(13)      & MIPL_Gain(12 downto 0);	
gain_ramp_2s	<= not Gain_Ramp(13)  	  & Gain_Ramp(12 downto 0);

-- Gate off Phase to Reference Error Until MIPL --------------------
ENABLE_MIPL_FEEDBACK:
process (clk, nreset, run, Ref_minus_Pherr)
begin
	if   (nreset = '0') then 
		ref_minus_pherr_2s <= (others => '0');
	elsif rising_edge(clk) then
		if   (run = '0') then 	
			ref_minus_pherr_2s <= (others => '0');
		else 
			ref_minus_pherr_2s <= (not Ref_minus_Pherr(31)) & Ref_minus_Pherr(30 downto 0);
		end if;
	end if;
end process;
		
	INITIAL_GAIN_RAMP : mult_32x14
	port map (
				dataa	 => ref_minus_pherr_2s(31 downto 0),
				datab	 => gain_ramp_2s(13 downto 0),
				result	 => ramp_2s(45 downto 0)
			 );
			   
	G1xFEEDBACK : mult_32x14 
	port map (
				dataa	 => ramp_2s(45) & ramp_2s(43 downto 13),
				datab	 => MIPL_Gain_2s(13 downto 0),
				result	 => product0_2s(45 downto 0)
			 );
		
	Output_2s(31 downto 0) <= product0_2s(45) & product0_2s(43 downto 13);		
	
	Monitor_Out(13 downto 0) <= NOT product0_2s(45) & product0_2s(42 downto 30);		
end main;

