
Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities



entity Ref_Curve_Adjust is
	port(clk        			: in std_logic;
		 nreset     			: in std_logic;
		 Update_Strobe			: in std_logic;
		 Use_Alt_Slope			: in std_logic;
		 Delta_T_at_Trig_2s		: in std_logic_vector(15 downto 0); 
		 Intvl_Offset_Gain_OB  	: in std_logic_vector(13 downto 0);
		 Base_Intvl_Clks		: in std_logic_vector(15 downto 8);
		 Alt_Slope_Offset		: in std_logic_vector(7 downto 0);
		 
		 Interval_Clks			: out std_logic_vector(15 downto 0);
		 Intvl_Clks_Mon_OB		: out std_logic_vector(13 downto 0)
		);
end Ref_Curve_Adjust;
	
architecture main of Ref_Curve_Adjust is

-- -------------------------------------------------------------
	signal upd					: std_logic;

	signal intvl_offset_gain_2s	: std_logic_vector(15 downto 0);
	signal product_2s			: std_logic_vector(31 downto 0);
	signal scaled_intvl_offs_2s	: std_logic_vector(15 downto 0);
	
	signal base_intvl_clks_2s	: std_logic_vector(15 downto 0);
	signal alt_slope_offset_2s	: std_logic_vector(15 downto 0);
	signal alt_slope_sum_2s		: std_logic_vector(15 downto 0);
	
	signal chosen_offset_2s		: std_logic_vector(15 downto 0);
	
	signal intvl_clks_sum_2s	: std_logic_vector(15 downto 0);
-------------------------------------------------------------

----------------------------------------------------------------
	component mult16signed
	PORT
	(
		dataa		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0)
	);
	end component;
	
	component addsub16signed
	PORT
	(
		add_sub		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (15 DOWNTO 0)
	);
	end component;
	
-- ## MAIN #########################################################
begin 
sync_UPDATE: led
port map(sig_in => Update_Strobe, clk => clk, clrn => nreset, pls_out => upd);


-- Convert Offset Binary Values to 2's Complement ------------------
intvl_offset_gain_2s	<= (15 downto 13 => not Intvl_Offset_Gain_OB(13)) 
											& Intvl_Offset_Gain_OB(12 downto 0);
base_intvl_clks_2s	<= X"00" & Base_Intvl_Clks; -- convert 8 bit unsigned int to 16 bit 2s
alt_slope_offset_2s	<= X"00" & Alt_Slope_Offset; -- convert 8 bit unsigned int to 16 bit 2s
	
	DELTA_T_x_GAIN : mult16signed 
	port map (
				dataa	 => Delta_T_at_Trig_2s,
				datab	 => intvl_offset_gain_2s,
				result	 => product_2s(31 downto 0)
			 );
			   	
	scaled_intvl_offs_2s <= product_2s(31) & product_2s(29 downto 15);
			   			 
	SUM_ALT_SLOPE_OFFSET : addsub16signed 
	port map (
				add_sub	 => '1',
				dataa	 => scaled_intvl_offs_2s(15 downto 0),
				datab	 => alt_slope_offset_2s(15 downto 0),
				result	 => alt_slope_sum_2s(15 downto 0)
			 );
			 
			 
	CHOOSE_ALT_SLOPE:
	process(clk, nreset, Use_Alt_Slope, alt_slope_sum_2s, scaled_intvl_offs_2s)
	begin
		if  (nreset = '0')       then chosen_offset_2s <= (others => '0');
		elsif rising_edge(clk)   then
			if   (Use_Alt_Slope = '1')  then 	
									chosen_offset_2s <= alt_slope_sum_2s;
			else	
									chosen_offset_2s <= scaled_intvl_offs_2s;
			end if;
		end if;
	end process;
	
	SUM_IN_BASE_OFFSET : addsub16signed 
	port map (
				add_sub	 => '1',
				dataa	 => base_intvl_clks_2s(15 downto 0),
				datab	 => chosen_offset_2s(15 downto 0),
				result	 => intvl_clks_sum_2s(15 downto 0)
			 );
			 
-- Syncronize Output ----------------------------

	process(clk, nreset, intvl_clks_sum_2s)
	begin
		if   (nreset = '0')    then 
			Interval_Clks <= (others => '0');
			Intvl_Clks_Mon_OB <= "10000000000000";
		elsif rising_edge(clk) then
			if   (upd = '1') then
				Interval_Clks <= intvl_clks_sum_2s;		
				Intvl_Clks_Mon_OB <= not intvl_clks_sum_2s(15) & intvl_clks_sum_2s(12 downto 0);
			end if;
		end if;
	end process;

end main;

